@@ -20,6 +20,12 @@
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
+/**
+ * @file
+ * Common code for Vorbis I encoder and decoder
+ * @author Denes Balatoni  ( dbalatoni programozo hu )
+ */
+
 #define ALT_BITSTREAM_READER_LE
 #include "avcodec.h"
 #include "get_bits.h"
@@ -117,8 +123,7 @@ int ff_vorbis_len2vlc(uint8_t *bits, uint32_t *codes, unsigned num)
     return 0;
 }
 
-int ff_vorbis_ready_floor1_list(AVCodecContext *avccontext,
-                                vorbis_floor1_entry *list, int values)
+void ff_vorbis_ready_floor1_list(vorbis_floor1_entry * list, int values)
 {
     int i;
     list[0].sort = 0;
@@ -142,11 +147,6 @@ int ff_vorbis_ready_floor1_list(AVCodecContext *avccontext,
     for (i = 0; i < values - 1; i++) {
         int j;
         for (j = i + 1; j < values; j++) {
-            if (list[i].x == list[j].x) {
-                av_log(avccontext, AV_LOG_ERROR,
-                       "Duplicate value found in floor 1 X coordinates\n");
-                return AVERROR_INVALIDDATA;
-            }
             if (list[list[i].sort].x > list[list[j].sort].x) {
                 int tmp = list[i].sort;
                 list[i].sort = list[j].sort;
@@ -154,10 +154,9 @@ int ff_vorbis_ready_floor1_list(AVCodecContext *avccontext,
             }
         }
     }
-    return 0;
 }
 
-static inline void render_line_unrolled(intptr_t x, int y, int x1,
+static inline void render_line_unrolled(intptr_t x, unsigned char y, int x1,
                                         intptr_t sy, int ady, int adx,
                                         float *buf)
 {
@@ -169,31 +168,31 @@ static inline void render_line_unrolled(intptr_t x, int y, int x1,
         if (err >= 0) {
             err += ady - adx;
             y   += sy;
-            buf[x++] = ff_vorbis_floor1_inverse_db_table[av_clip_uint8(y)];
+            buf[x++] = ff_vorbis_floor1_inverse_db_table[y];
         }
-        buf[x] = ff_vorbis_floor1_inverse_db_table[av_clip_uint8(y)];
+        buf[x] = ff_vorbis_floor1_inverse_db_table[y];
     }
     if (x <= 0) {
         if (err + ady >= 0)
             y += sy;
-        buf[x] = ff_vorbis_floor1_inverse_db_table[av_clip_uint8(y)];
+        buf[x] = ff_vorbis_floor1_inverse_db_table[y];
     }
 }
 
-static void render_line(int x0, int y0, int x1, int y1, float *buf)
+static void render_line(int x0, unsigned char y0, int x1, int y1, float *buf)
 {
     int dy  = y1 - y0;
     int adx = x1 - x0;
     int ady = FFABS(dy);
     int sy  = dy < 0 ? -1 : 1;
-    buf[x0] = ff_vorbis_floor1_inverse_db_table[av_clip_uint8(y0)];
+    buf[x0] = ff_vorbis_floor1_inverse_db_table[y0];
     if (ady*2 <= adx) { // optimized common case
         render_line_unrolled(x0, y0, x1, sy, ady, adx, buf);
     } else {
-        int base  = dy / adx;
-        int x     = x0;
-        int y     = y0;
-        int err   = -adx;
+        int base = dy / adx;
+        int x    = x0;
+        unsigned char y = y0;
+        int err  = -adx;
         ady -= FFABS(base) * adx;
         while (++x < x1) {
             y += base;
@@ -202,7 +201,7 @@ static void render_line(int x0, int y0, int x1, int y1, float *buf)
                 err -= adx;
                 y   += sy;
             }
-            buf[x] = ff_vorbis_floor1_inverse_db_table[av_clip_uint8(y)];
+            buf[x] = ff_vorbis_floor1_inverse_db_table[y];
         }
     }
 }
